"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var ServiceSpec = require("../spec/service");
var fs = require("fs-extra");
var pathModule = require("path");
var PersistantService = /** @class */ (function (_super) {
    __extends(PersistantService, _super);
    function PersistantService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.storageName = "persistant.json";
        _this.storagePath = pathModule.join(_this.settings.persistant.storage, _this.storageName);
        _this.datas = {};
        _this.tracks = {};
        return _this;
    }
    PersistantService.prototype.initialize = function () {
        var _this = this;
        this.services.GracefulShutdownService.graceful({
            name: "persistant",
            timeout: 1000 * 30
        }, function (done) {
            _this.save();
            console.log("Persisted");
            done();
        });
        fs.mkdirpSync(this.settings.persistant.storage);
        if (!fs.existsSync(this.storagePath)) {
            return;
        }
        try {
            this.datas = JSON.parse(fs.readFileSync(this.storagePath, "utf8"));
        }
        catch (e) {
            console.error("Fail to load broken persistant files");
            return;
        }
    };
    PersistantService.prototype.track = function (obj) {
        if (this.tracks[obj.name]) {
            console.error("Can't duplicate tracking " + obj.name);
            return false;
        }
        this.tracks[obj.name] = obj;
    };
    PersistantService.prototype.restore = function (obj) {
        var data = this.datas[obj.name];
        if (!data) {
            console.log("No restore for", obj.name);
            return false;
        }
        obj.fromJSON(data);
    };
    PersistantService.prototype.save = function () {
        for (var name_1 in this.tracks) {
            var obj = this.tracks[name_1];
            this.datas[name_1] = obj.toJSON();
        }
        fs.writeFileSync(this.storagePath, JSON.stringify(this.datas));
    };
    return PersistantService;
}(ServiceSpec.PersistantService));
exports.PersistantService = PersistantService;
//# sourceMappingURL=persistantService.js.map